SUBROUTINE compute_coupling_matrix(nlmax,nbmax,binsize,M_l,K,ipiv)
!
! This subroutine calculates an LU decomposition of the coupling
! matrix. Ref: Hivon et al., ApJ, 567, 2 (2002) and a binned l(l+1)cl/twopi
! version
!
! = Inputs =
! 1. nlmax (maximum multipole) [integer]
! 2. M_l(0:lmax) [power spectrum of a mask; double precision]
! 3. nbmax (number of bins) [integer]
! 4. binsize (size of the bin) [integer]
!
! = Outputs =
! 1. K(1:nbmax,1:nbmax) [double precision]
! 2. ipiv(1:lmax-1) [integer]
!
! based on June 27, 2010: E.Komatsu
! modified for binned MASTER April 17, 2013: R.Flauger
!
! Calculation of the couplong matrix is modified to use symmetry of the square of Wigner 3j
! July 26, 2018: R. Makiya
!
  implicit none
  integer,intent(IN) :: nlmax,nbmax,binsize
  double precision, intent(INOUT) :: M_l(0:nlmax)
  double precision, dimension(:,:), allocatable :: Ftp
  double precision, intent(OUT) :: K(nbmax,nbmax)
  integer, intent(OUT) :: ipiv(nbmax)
  integer :: info,l,l2,l3,lmin,lmax,b1,b2
  double precision :: el1min,el1max
  double precision :: thrcof(2*nlmax+2),w3j(0:nlmax)
  double precision, parameter :: pi= 3.14159265358979323846264d0
  allocate(Ftp(1:nlmax-1,1:nlmax-1))
  do l=0,nlmax
     M_l(l)=(2d0*dble(l)+1d0)*M_l(l)
  enddo
! compute TT-TT coupling
  Ftp=0d0
  do l2=2,nlmax
     do l3=l2,nlmax
        CALL DRC3JJ(dble(l2),dble(l3),0.d0,0.d0,EL1MIN,EL1MAX, &
             THRCOF,2*nlmax+2,info)
        lmin=INT(EL1MIN)
        lmax=MIN(nlmax,INT(EL1MAX))
        w3j(lmin:lmax)=thrcof(1:lmax-lmin+1)**2d0
        Ftp(l2-1,l3-1)=SUM(M_l(lmin:lmax)*w3j(lmin:lmax)) 
        if (l3 /= l2) then
          Ftp(l3-1,l2-1) = Ftp(l2-1,l3-1)
        endif
     enddo
  enddo
  do l3=2,nlmax
    Ftp(1:nlmax-1,l3-1) = (2d0*l3+1d0)/(4d0*pi)*Ftp(1:nlmax-1,l3-1)
  end do
! compute binned coupling matrix for l(l+1)cl/twopi
  K=0d0
  do b2=1,nbmax
     do b1=1,nbmax
        do l3=2+(b2-1)*binsize,2+b2*binsize-1
           do l2=2+(b1-1)*binsize,2+b1*binsize-1
              K(b1,b2)=K(b1,b2)+dble(l2)*(dble(l2)+1d0) &
                               /dble(l3)/(dble(l3)+1d0) &
                               *Ftp(l2-1,l3-1)/dble(binsize)
           enddo
        enddo
     enddo
  enddo
  deallocate(Ftp)
! LU decompose the binned coupling matrix
  CALL DGETRF(nbmax,nbmax,K,nbmax,ipiv,info)
  if(info.ne.0)then
     print*,' error DGETRF: info =',info
     stop
  endif
  RETURN
END SUBROUTINE compute_coupling_matrix
!-------------------------------------------------------------------------
SUBROUTINE COMPUTE_CL(nlmax,nbmax,binsize,cl,K,ipiv,cb)
!
! This subroutine solves the following linear equation:
!
!    K * C_b = \tilde{C}_b
!
! for C_b, where C_b contains a factor l(l+1)/twopi. I.e., binning is done
! on l(l+1)cl/twopi.
!
! The input K is an LU decomposition of the original matrix 
! computed by DGETRF.
!
! = Inputs =
! 1. nlmax [integer]
! 2. nbmax [integer]
! 3. binsize [integer]
! 2. cl(0:nlmax) [double precision] WITHOUT l(l+1)/twopi
! 3. K(0:nbmax-1,0:nbmax-1) [double precision]
! 4. ipiv(1:nbmax) [integer]
!
! = Output =
! 1. cb(1:nbmax) [double precision] binned l(l+1)cl/twopi
!
! based on December 18, 2003: E.Komatsu
! modified for binned MASTER April 17, 2013: R.Flauger
  IMPLICIT NONE
  integer, INTENT(IN) :: nlmax,nbmax,binsize
  double precision, INTENT(IN) :: K(nbmax,nbmax)
  integer, INTENT(IN) :: ipiv(nbmax)
  double precision, INTENT(IN) :: cl(0:nlmax)
  double precision, INTENT(OUT) :: cb(1:nbmax)
  double precision, ALLOCATABLE :: cls(:)
  integer :: info,ll,b
  double precision, parameter :: pi= 3.14159265358979323846264d0
  ALLOCATE(cls(nbmax))
  cls=0d0
  do b=1,nbmax
    do ll=2+(b-1)*binsize,2+b*binsize-1
       ! binning l(l+1)cl/twopi
      cls(b)=cls(b)+dble(ll)*(dble(ll)+1d0)/2d0/pi*cl(ll)/binsize 
    enddo
  enddo
  CALL DGETRS('N',nbmax,1,K,nbmax,ipiv,cls,nbmax,info)
  if(info.ne.0)then
     print*,' error at DGETRS in COMPUTE_CL : info = ',info
  endif
  cb(1:nbmax)=cls(1:nbmax)
  DEALLOCATE(cls)
  RETURN
END SUBROUTINE COMPUTE_CL
